/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTController_inl__
#define __RTController_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActorRef_h__
#include <RTActorRef.h>
#endif

#ifndef __RTController_h__
#include <RTController.h>
#endif

#ifndef __RTMessage_h__
#include <RTMessage.h>
#endif

RTS_INLINE RTController::Error RTController::getError( void ) const
{
    return errorNumber;
}

#if OTRTSDEBUG == DEBUG_NONE
RTS_INLINE void RTController::setError( Error        error,
					Primitive,
					const void *,
					const void * )
{
    errorNumber = error;
}
#endif

#if OTRTSDEBUG < DEBUG_VERBOSE
RTS_INLINE void RTController::trace( Primitive, const void *, const void * )
{
}
#endif

RTS_INLINE RTDebugger * RTController::debugger( void )
{
#if OTRTSDEBUG == DEBUG_NONE
    return (RTDebugger *)0;
#else
    return _debugger;
#endif
}

#if ! RTS_COUNT
RTS_INLINE void RTController::countActorDestroy   ( void        ) {}
RTS_INLINE void RTController::countActorIncarnate ( void        ) {}
RTS_INLINE void RTController::countMessageTransfer( RTMessage * ) {}
RTS_INLINE void RTController::countPortDestroy    ( int         ) {}
RTS_INLINE void RTController::countPortIncarnate  ( int         ) {}
RTS_INLINE void RTController::countTimerCancel    ( void        ) {}
RTS_INLINE void RTController::countTimerExpire    ( void        ) {}
RTS_INLINE void RTController::countTimerStart     ( void        ) {}
#endif

RTS_INLINE RTMessage * RTController::getReplyBuffer( void )
{
    return replyBuffer;
}

RTS_INLINE void RTController::restoreReplyBuffer( RTMessage * oldBuffer )
{
    replyBuffer = oldBuffer;
}

RTS_INLINE RTMessage * RTController::setReplyBuffer( RTMessage * newBuffer )
{
    RTMessage * oldBuffer = replyBuffer;

    replyBuffer = newBuffer;

    return oldBuffer;
}

// get a free message
RTS_INLINE RTMessage * RTController::newMsg( void )
{
    RTMessage * msg;

#if PURIFY

    msg = new RTMessage;

#if RTS_COUNT
    if( msg != (RTMessage *)0 )
    {
	counts.messagesAllocated += 1U;
	if( counts.peakMessagesAllocated < counts.messagesAllocated )
	    counts.peakMessagesAllocated = counts.messagesAllocated;
    }
#endif

#else // PURIFY

    for(;;)
    {
	if( (msg = freeList) == (RTMessage *)0 )
	{
	    unsigned allocCount = resourceMgr.msgAlloc( freeList );

	    if( allocCount == 0U )
		break;

	    freeListSize += allocCount;

#if RTS_COUNT
	    counts.messagesAllocated += allocCount;
	    if( counts.peakMessagesAllocated < counts.messagesAllocated )
		counts.peakMessagesAllocated = counts.messagesAllocated;
#endif

	    msg = freeList;
	}

	freeList      = msg->next;
	freeListSize -= 1U;
	break;
    }

#endif // PURIFY

    return msg;
}

RTS_INLINE void RTController::freeMsg( RTMessage * msg )
{
#if PURIFY

#if RTS_COUNT
    counts.messagesAllocated -= 1U;
#endif

    delete msg;

#else // PURIFY

    const unsigned maxFreeListSize = 100U;
    const unsigned minFreeListSize =  20U;

    msg->clear();

    msg->next = freeList;
    freeList  = msg;

    if( ++freeListSize > maxFreeListSize )
    {
#if RTS_COUNT
	counts.messagesAllocated -= freeListSize - minFreeListSize;
#endif

	resourceMgr.msgFree( freeList, freeListSize - minFreeListSize );
	freeListSize = minFreeListSize;
    }
#endif // PURIFY
}

RTS_INLINE void RTController::receive( RTMessage * msg, RTController * sender )
{
#if ! USE_THREADS
    (void)sender; // it _is_ used
#else
    if( sender == this )
#endif
    {
	unsigned char priority = msg->priority;

	internalBits |= (short)( 1 << priority );

	internalQ[ priority ].nqTail( msg );
    }
#if USE_THREADS
    else
    {
	// It is safe to modify the sender because it is within
	// the context of that thread that we are executing.
	sender->countMessageTransfer( msg );

	peer_receive( msg );
    }
#endif
}

#endif // __RTController_inl__
