/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTcActors_h__
#define __RTcActors_h__ included

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

/*
 * This file contains the public ROOM interface for C actors.
 */

#define RSLPANIC      Panic
#define RSLHIGH       High
#define RSLGENERAL    General
#define RSLLOW        Low
#define RSLBACKGROUND Background

/*
 * In the following macros the arguments have the following meanings
 *
 * d    data to be sent in a message
 * p    port, SAP or SPP reference name
 * i    port index
 * s    signal name
 * t    timeout
 * u    priority (urgency)
 */

#define ROOM_CancelTimer(p,id)          ((p).cancelTimer((id)))
#define ROOM_InSignal(p,s)              (s)
#define ROOM_InformIn(p,t)              ((p).informIn((t)))
#define ROOM_InformInData(p,t,d)        ((p).informIn((t),(d)))
#define ROOM_IsTimerValid(id)           ((id).isValid())
#define ROOM_MessageData()              (msg->data)
#define ROOM_MessageDataPointer(type)   (POINTER_OF(type)msg->data)
#define ROOM_MessageDefer()             (msg->defer())
#define ROOM_MessageIndex()             (msg->sapIndex0())
#define ROOM_MessageSap()               (*msg->sap())
#define ROOM_MessageSignal()            (msg->signal)
#define ROOM_PortIndexedSend(p,i,s)     ((p)[(i)]->send((s)))
#define ROOM_PortIndexedSendData(p,i,s,d) \
					((p)[(i)]->send((s),(d)))
#define ROOM_PortSend(p,s)              ((p).send((s)))
#define ROOM_PortSendData(p,s,d)        ((p).send((s),(d)))
#define ROOM_PrioritizedPortSend(p,s,u) ((p).send((s),(void*)0,(u)))
#define ROOM_PrioritizedPortIndexedSend(p,i,s,u) \
					((p)[(i)]->send((s),(void*)0,(u)))
#define ROOM_PrioritizedPortIndexedSendData(p,i,s,u,d) \
					((p)[(i)]->send((s),(d),(u)))
#define ROOM_PrioritizedPortSendData(p,s,u,d) \
					((p).send((s),(d),(u)))
#define ROOM_RecallAll(p)               ((p).recallAll())
#define ROOM_SAPDeRegister(p)           ((p).deregister())
#define ROOM_SAPRegister(p,name)        ((p).registerAs((name)))
#define ROOM_SPPDeRegister(p)           ((p).deregister())
#define ROOM_SPPRegister(p,name)        ((p).registerAs((name)))
#define ROOM_Signal(p,s)                (s)
#define RSL_Timeout()                   (timeout)

#endif // __RTcActors_h__
