
/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTTimerActor.h>
#include <RTTiming.h>

void RTActor::_predestroy( void )
{
    const RTActor_class    * info  = getActorData();
    RTTimerActor           * timer = RTTimerActor::soleInstance();
    const RTPortDescriptor * port;
    int                      count;
	int					     i;
    RTProtocol             * end;

    // disable binding notifications: we will soon purge any rtUnbound messages
    for( port = info->ports, count = info->numPorts; --count >= 0; ++port )
    {
	if( ( port->properties & RTPortDescriptor::MaskKind )
			      != RTPortDescriptor::KindSpecial )
	{
	    end = RTObjectOf( RTProtocol, port->offset, this );

	    end->bindingNotification( 0 );
	}
    }

    // destroy or deport all capsule roles
    const RTComponentDescriptor * component;

    count     = info->numComponents;
    component = &info->components[ 0 ];

    i = 0;

    while( i++ < count )
    {
		RTActorRef * ref = RTObjectOf( RTActorRef, component->offset, this );

		if( ref->isImported() != 0 )
			ref->deport();
		else
			(void)ref->destroy();

		component++;
    }

    // deregister all SAPs and SPPs, cancel any outstanding timer requests
    for( port = info->ports, count = info->numPorts; --count >= 0; ++port )
    {
	end = RTObjectOf( RTProtocol, port->offset, this );

	switch( port->properties & RTPortDescriptor::MaskKind )
	{
	case RTPortDescriptor::KindUnwired:
	    if( end->isRegistered() == 0 )
		break;

	    if( ( end->getFlags() & RTProtocol::RegisteredAsSAP ) != 0 )
		(void)end->deregisterSAP( RTProtocol::Automatic );
	    else
		(void)end->deregisterSPP( RTProtocol::Automatic );
	    break;

	case RTPortDescriptor::KindSpecial:
	    if( port->protocol == &Timing::Base::rt_class )
	    {
		// cancel all outstanding timer requests
		if( timer != (RTTimerActor *)0 )
		    timer->cancelAll( (Timing::Base *)end );
	    }
	    break;

	case RTPortDescriptor::KindWired:
	default:
	    break;
	}
    }
}
