/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTInterfaceDescriptor.h"
#pragma implementation "RTLocalBindingDescriptor.h"
#pragma implementation "RTPortDescriptor.h"
#pragma implementation "RTSpecialPorts.h"
#pragma implementation "RTSpecialSignals.h"
#endif

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTFieldOffset.h>
#include <RTInterfaceDescriptor.h>
#include <RTMemoryUtil.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTRelayDescriptor.h>
#include <RTRelayPort.h>

void RTActor::_preinitialize( int                           _homeIndex,
			      const RTComponentDescriptor * _home )
{
    homeIndex = _homeIndex;

    const RTActor_class * info = getActorData();

    {
	int numStates    = info->numStates;
	int historyBytes = numStates;

#if RTStateId_MaxSize != 1
	int idShift;

	if( numStates < 0x100 )
	    idShift = 0; // state numbers fit in 8 bits
	else
	{
	    if( numStates < 0x10000 )
		idShift = 1; // state numbers fit in 16 bits
	    else
		idShift = 2; // state numbers need 32 bits

	    historyBytes <<= idShift; //lint !e701
	}
#endif

	RTFieldOffset allocSize;

	allocSize  = RTOffsetOf( State, history );
	allocSize += (RTFieldOffset)sizeof( State );
	allocSize -= (RTFieldOffset)1;
	allocSize += (RTFieldOffset)historyBytes;

	state = new State [ (unsigned)allocSize / sizeof( State ) ];

	state->current = (RTStateId)1;
#if RTStateId_MaxSize != 1
	state->idShift = (unsigned char)idShift;
#endif
	RTMemoryUtil::memset( &state->history, 0, historyBytes ); //lint !e669
    }

    {
	const RTActorClass * type      = info->type;
	int                  numRelays = type->numRelays;

	if( numRelays != 0 )
	{
	    int                           numIfaces = _home->numInterfaces;
	    const RTInterfaceDescriptor * iface     = _home->interfaces;
	    RTRelayPort                 * relay;
	    const RTRelayDescriptor     * relayDesc = type->relays;

	    relays = new RTRelayPort [ (unsigned)numRelays ];

	    for( relay = relays, relayDesc = type->relays;
		 --numRelays >= 0;
		 ++relay,        ++relayDesc )
	    {
		int used;

		if( numIfaces <= 0 )
		    used = 0;
		else
		{
		    --numIfaces;
		    used = iface++->replication;
		}
		relay->init( relayDesc->replication - used );
	    }
	}
    }

    const RTPortDescriptor * port = info->ports;
    int                      i;

    for( i = info->numPorts; --i >= 0; ++port )
    {
	RTProtocol * end = RTObjectOf( RTProtocol, port->offset, this );

	end->init( port );
    }

    const RTComponentDescriptor * component = info->components;

    for( i = info->numComponents; --i >= 0; ++component )
    {
	RTActorRef * ref = RTObjectOf( RTActorRef, component->offset, this );

	ref->init( component );
    }
}
