/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActorProbe.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTTiming.h>

void RTActorProbe::evtMessageOutIn( const RTMessage         * msg,
				    RTEventInfo::SendMethod   sendMethod )
{
    RTEventInfo * eventInfo = new RTEventInfo( RTEventInfo::EventMsgOutIn,
					       _id,
					       _actorIndex );

    eventInfo->message    = new RTMessage( *msg );
    eventInfo->sendMethod = sendMethod;

    RTProtocol * sap = msg->sap();

    if( sap == (RTProtocol *)0 )
    {
	switch( sendMethod )
	{
	case RTEventInfo::SendInformAt:
	case RTEventInfo::SendInformEvery:
	case RTEventInfo::SendInformIn:
	    // This is a faked message; it's a representation of calling
	    // informIn/At/Every. Change event kind to EventTimer.
	    eventInfo->eventKind = RTEventInfo::EventTimer;
	    break;

#ifdef _lint
	case RTEventInfo::SendNormal:
	case RTEventInfo::SendInvoke:
	case RTEventInfo::SendReply:
#endif
	default:
	    break;
	}
    }
    else if( sap->getInfo()->protocol == &Timing::Base::rt_class )
    {
	// This is a real message, but it's a timeout, so we'll change
	// the event kind to EventTimer to make it easier for the MSCs.
	// Set the send timestamp to the current time.
	eventInfo->eventKind = RTEventInfo::EventTimer;
	_debugger->setTimestamp( eventInfo->message, (const RTProtocol *)0, 0 );
    }

    _debugger->getTimestamp( eventInfo->timestamp );

    context()->breakPoint( _actor, eventInfo );
}
