/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDataObject.h>
#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void pRTDataObject_init( const RTObject_class *, void * target )
{
    *(RTDataObject **)target = (RTDataObject *)0;
}

static void pRTDataObject_copy( const RTObject_class *,
				void                 * target,
				const void           * source )
{
    RTDataObject * object = *(RTDataObject * const *)source;

    if( object != (RTDataObject *)0 )
	object = object->copy();

    *(RTDataObject **)target = object;
}

#if OBJECT_DECODE
static int pRTDataObject_decode( const RTObject_class *,
				 void                 * target,
				 RTDecoding           * coding )
{
    void                 * object;
    const RTObject_class * type;

    if( coding->get( &object, &type ) == 0 )
	return 0;

    if( type->isKindOf( &RTType_RTDataObject ) == 0 )
    {
	type->destroy( object, RTObject_class::DestroyAndDeallocate );
	return 0;
    }

    *(RTDataObject **)target = (RTDataObject *)object;

    return 1;
}
#endif

#if OBJECT_ENCODE
static int pRTDataObject_encode( const RTObject_class *,
				 const void           * source,
				 RTEncoding           * coding )
{
    const RTDataObject * object = *(RTDataObject * const *)source;

    if( object == (const RTDataObject *)0 )
	object = &EmptyObject;

    return coding->put( (const void *)object, object->getClassData() );
}
#endif

static void pRTDataObject_destroy( const RTObject_class *, void * target )
{
    RTDataObject * & object = *(RTDataObject **)target;

    delete object;

    object = (RTDataObject *)0;
}

const RTObject_class RTType_RTpRTDataObject =
{
    (const RTObject_class *)0,
    (RTSuperAccessFunction)0,
    (const char *)0, // "(RTDataObject *)",
    (RTVersionId)0,
    (RTFieldOffset)sizeof( RTDataObject * ),
    pRTDataObject_init,
    pRTDataObject_copy,
#if OBJECT_DECODE
    pRTDataObject_decode,
#endif
#if OBJECT_ENCODE
    pRTDataObject_encode,
#endif
    pRTDataObject_destroy,
    0, (const RTFieldDescriptor *)0
};
