/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTProbe.h>

int RTDebugger::deleteProbe( int probeId )
{
    int result = probeId;

#if USE_THREADS
    _mutex->enter();
#endif

    RTProbe * probe = getProbe( probeId );

    if( probe == (RTProbe *)0 )
	result = -1;
    else if( probe->adjustRefCount( -1 ) <= 0 ) // no one left
    {
	for( int i = 0; i < parkedProbes.size(); ++i )
	{
	    if( parkedProbes[ i ] == (RTProbe *)probe )
	    {
		parkedProbes[ i ] = (void *)0;
		--nrParkedProbes;
		break;
	    }
	}

	RTActor * actor = probe->actor();

	if( actor == (RTActor *)0 )
	{
	    // No actor connected to this probe; it is safe to
	    // disconnect and delete the probe now.
	    (void)probe->disconnectProbe( (RTActor *)0 );
	    delete probe;
	}
	else
	{
	    // Probe is connected to an actor; mark probe for
	    // destruction and ask the actor to take care of it.
	    probe->_destroyMe = 1;
	    sendProbeSignal( actor, System );
	}

	probes[ probeId ] = (void *)0;
    }

#if USE_THREADS
    _mutex->leave();
#endif

    return result;
}
