/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTBindingEnd.h>
#include <RTController.h>
#include <RTDiag.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTProtocolAdapter.h>

static RTProtocolAdapter adapter;

void RTProtocol::bindAt( int            index,
			 RTBindingEnd & peer,
			 RTController * initiator )
{
    RTProtocol * peerPort  = peer.port;
    int          peerIndex = peer.index;

    RTAssert( 0 <=     index &&     index <           _info->replication );
    RTAssert( 0 <= peerIndex && peerIndex < peerPort->_info->replication );

    End * local  = &          _ends[     index ];
    End * remote = &peerPort->_ends[ peerIndex ];

    // The local and remote ports should both be in the same state.
    RTAssert( local->state == remote->state );

    if( local->state != End::Unbound )
    {
	// This is normal. Incarnating an actor which forms the final
	// link in a binding between to pre-existing ports results in
	// two calls to this function with the roles reversed.
	// Just verify that this is the case and return.

	RTAssert( local ->remote      == peerPort );
	RTAssert( local ->remoteIndex == peerIndex );
	RTAssert( remote->remote      == this  );
	RTAssert( remote->remoteIndex == index );
	return;
    }

    RTController * thisContext = this    ->context();
    RTController * peerContext = peerPort->context();

    local ->remote      = peerPort;
    local ->remoteIndex = peerIndex;
    remote->remote      = this;
    remote->remoteIndex = index;

    adapter.adapt( _info->protocol,   peerPort->_info->protocol,
		   &local->signalMap, &remote->signalMap );

    End::State state = thisContext == peerContext ? End::Bound
						  : End::BoundXThread;

    local ->state = state;
    remote->state = state;

    if( initiator != (RTController *)0 )
    {
	if( bindingNotificationRequested() != 0 )
	    notifyBoundAt( index, initiator );

	if( peerPort->bindingNotificationRequested() != 0 )
	    peerPort->notifyBoundAt( peerIndex, initiator );
    }
}
