/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTTimerList.h>
#include <RTTimerNode.h>

void RTTimerList::remove( RTTimerNode * node )
{
    unsigned index = node->link.index;

    if( index < used && nodes[ index ] == node )
    {
	node = nodes[ --used ];
	nodes[ node->link.index = index ] = node;

	bool moved_up = false;

	while( index != 0U )
	{
	    unsigned      parent_index = ( index - 1U ) >> 1;
	    RTTimerNode * parent_node  = nodes[ parent_index ];

	    if( node->expireTime >= parent_node->expireTime ) //lint !e909
		break;

	    // swap node & parent_node
	    nodes[ node       ->link.index = parent_index ] = node;
	    nodes[ parent_node->link.index =        index ] = parent_node;
	    moved_up = true;
	    index    = parent_index;
	}

	if( moved_up )
	{
	    // We moved the node up in the tree. We don't need to consider
	    // swaps with its new children because they were properly below
	    // a node which is now below the relocated node.
	    return;
	}

	unsigned child_index;
	unsigned right_index;

	for( ; (child_index = ( index << 1 ) + 1U) < used; index = child_index )
	{
	    RTTimerNode * child_node = nodes[ child_index ];

	    if( (right_index = child_index + 1U) < used )
	    {
		RTTimerNode * right_node = nodes[ right_index ];

		// we want to compare with the smaller child node
		if( right_node->expireTime <=
		    child_node->expireTime ) //lint !e909
		{
		    child_node  = right_node;
		    child_index = right_index;
		}
	    }

	    if( node->expireTime <= child_node->expireTime ) //lint !e909
		break;

	    // swap node & child_node
	    nodes[ node      ->link.index = child_index ] = node;
	    nodes[ child_node->link.index =       index ] = child_node;
	}
    }
}
