/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTFormat.h>
#include <RTMemoryUtil.h>
#include <RTPriority.h>

#include <endePriv.h>

extern "C"
{

/*----------------------------------------------------------------------------
|  Common lookup table:
+---------------------------------------------------------------------------*/

/* Generic no-parameter decode: */
int decNoParams( TO_bufferInfo * buf, TO_structInfo * )
{
    return buf->status = DEC_SUCCESS;
}

/* Generic no-parameter free: */
void freeNoParams( TO_structInfo * )
{
}

struct lookupTable
{
    const char * funcName;  /* string with function name    */
    decoFunc     decoPtr;   /* pointer to decoding function */
    freeFunc     freePtr;   /* pointer to freeing function  */
    unsigned     dataSize;  /* size of the data field       */
};

static const lookupTable theTable[ FUNC_LAST ] =
{
/*    func string      decoding     freeing       size of data chunk */
    { "",              decNoParams, freeNoParams, 0U                         },
    { "resultCode",    decResultCo, freeNoParams, sizeof( TO_resultCode    ) },
    { "evtTargetInfo", decEvtTargI, freeEvtTargI, sizeof( TO_evtTargetInfo ) },
    { "evtPresence",   decEvtPrese, freeEvtPrese, sizeof( TO_evtPresence   ) },
    { "evtReference",  decEvtRefer, freeEvtRefer, sizeof( TO_evtReference  ) },
    { "evtChain",      decEvtChain, freeEvtChain, sizeof( TO_evtChain      ) },
    { "evtVariable",   decEvtVaria, freeEvtVaria, sizeof( TO_evtVariable   ) },
    { "evtMsgOut",     decEvtMsgOu, freeEvtMsgOu, sizeof( TO_evtMsgOut     ) },
    { "evtMsgIn",      decEvtMsgIn, freeEvtMsgIn, sizeof( TO_evtMsgIn      ) },
    { "evtMsgOutIn",   decEvtMsgOI, freeEvtMsgOI, sizeof( TO_evtMsgOutIn   ) },
    { "evtRtsStatus",  decEvtRtsSt, freeNoParams, sizeof( TO_evtRtsStatus  ) },
    { "evtHistory",    decEvtHisto, freeEvtHisto, sizeof( TO_evtHistory    ) },
    { "requestInfo",   decNoParams, freeNoParams, 0U                         },
    { "closeSession",  decNoParams, freeNoParams, 0U                         },
    { "createDaemon",  decCmdCreDa, freeCmdDaemo, sizeof( TO_daemonSett    ) },
    { "updateDaemon",  decCmdUpdDa, freeCmdDaemo, sizeof( TO_daemonSett    ) },
    { "commandDaemon", decCmdCmdDa, freeCmdDaemo, sizeof( TO_daemonSett    ) },
    { "refreshDaemon", decCmdRefDa, freeCmdDaemo, sizeof( TO_daemonSett    ) },
    { "destroyDaemon", decNoParams, freeNoParams, 0U                         },
    { "addActor",      decCmdAddAc, freeCmdActor, sizeof( TO_actorInfo     ) },
    { "delActor",      decCmdDelAc, freeCmdActor, sizeof( TO_actorInfo     ) },
    { "addPort",       decCmdAddPo, freeCmdPort,  sizeof( TO_portInfo      ) },
    { "delPort",       decCmdDelPo, freeCmdPort,  sizeof( TO_portInfo      ) },
    { "addState",      decCmdAddSt, freeCmdState, sizeof( TO_stateInfo     ) },
    { "delState",      decCmdDelSt, freeCmdState, sizeof( TO_stateInfo     ) },
    { "addChain",      decCmdAddCh, freeCmdChain, sizeof( TO_chainInfo     ) },
    { "delChain",      decCmdDelCh, freeCmdChain, sizeof( TO_chainInfo     ) },
    { "addVar",        decCmdAddVa, freeCmdAdDeV, sizeof( TO_variableInfo  ) },
    { "delVar",        decCmdDelVa, freeCmdAdDeV, sizeof( TO_variableInfo  ) },
    { "setVar",        decCmdSetVa, freeCmdSeGeV, sizeof( TO_variableData  ) },
    { "getVar",        decCmdGetVa, freeCmdSeGeV, sizeof( TO_variableData  ) },
    { "injectMsg",     decCmdInjec, freeCmdInjec, sizeof( TO_injectMsg     ) }
};

/*----------------------------------------------------------------------------
|  Common decoding functions:
+---------------------------------------------------------------------------*/

/*--- Basic decoding: ------------------------------------------------------*/

static int doDecodeFunc( TO_bufferInfo * buf, int * pFunc )
{
    if( buf->size <= 5 ||
	RTMemoryUtil::memcmp( buf->data, "FUNC_", 5 ) != 0 )
    {
	return buf->status = DEC_NO_FUNC_ID;
    }

    buf->used = 5;

    char funcName[ 20 ];

    for( int i = 0; ; ++i, ++buf->used )
    {
	if( buf->used >= buf->size ||
	    i         >= (int)sizeof( funcName ) - 1 ||
	    (funcName[ i ] = buf->data[ buf->used ]) == ' ' )
	{
	    funcName[ i ] = '\0';
	    break;
	}
    }

    for( int func = FUNC_NONE; ++func < FUNC_LAST; )
    {
	if( RTMemoryUtil::strcmp( funcName, theTable[ func ].funcName ) == 0 )
	{
	    *pFunc = func;
	    return DEC_SUCCESS;
	}
    }

    return buf->status = DEC_BAD_FUNC_TYPE;
}

int doDecodeInt( TO_bufferInfo * buf, int * pIntValue )
{
    if( buf->used + 5 >= buf->size ||
	RTMemoryUtil::memcmp( buf->data + buf->used, " INT ", 5 ) != 0 )
    {
	return buf->status = DEC_NO_INT_ID;
    }

    buf->used += 5;

    char intStr[ 20 ];

    for( int i = 0; ; ++i, ++buf->used )
    {
	if( buf->used >= buf->size ||
	    i         >= (int)sizeof( intStr ) - 1 ||
	    (intStr[ i ] = buf->data[ buf->used ]) == ' ' )
	{
	    intStr[ i ] = '\0';
	    break;
	}
    }

    *pIntValue = RTMemoryUtil::atoi( intStr );

    return DEC_SUCCESS;
}

int doDecodeStr( TO_bufferInfo * buf, char ** pStrValue )
{
    *pStrValue = (char *)0;

    if( buf->used + 5 >= buf->size ||
	RTMemoryUtil::memcmp( buf->data + buf->used, " STR ", 5 ) != 0 )
    {
	return buf->status = DEC_NO_STR_ID;
    }

    buf->used += 5;

    char intStr[ 20 ];

    for( int i = 0; ; ++i, ++buf->used )
    {
	if( buf->used >= buf->size ||
	    i         >= (int)sizeof( intStr ) - 1 )
	{
	    return buf->status = DEC_NO_STR_SPC;
	}

	if( (intStr[ i ] = buf->data[ buf->used ]) == ' ' )
	{
	    intStr[ i ] = '\0';
	    break;
	}
    }

    int nrChars = RTMemoryUtil::atoi( intStr );

    if( ++buf->used + nrChars > buf->size )
	return buf->status = DEC_UNDERRUN;

    *pStrValue = RTMemoryUtil::strdup( buf->data + buf->used, nrChars );

    buf->used += nrChars;

    return DEC_SUCCESS;
}

/*--- Semi-special decoding: -----------------------------------------------*/

static int doDecodeDaemonInfo( TO_bufferInfo * buf, TO_daemonInfo * daemon )
{
    if( daemon == (TO_daemonInfo *)0 )
	return buf->status = DEC_BAD_PARM;

    if( doDecodeInt( buf, &daemon->number   ) != DEC_SUCCESS ||
	doDecodeStr( buf, &daemon->invokeID ) != DEC_SUCCESS )
    {
	return buf->status;
    }

    return DEC_SUCCESS;
}

/*--- Decoding data struct fiddling: ---------------------------------------*/

void * initStructInfo( TO_structInfo * info, int func )
{
    if( info == (TO_structInfo *)0 )
	return (void *)0;

    if( func <= FUNC_NONE || FUNC_LAST <= func )
	return (void *)0;

    info->func = func;

    unsigned dataSize;

    if( (dataSize = theTable[ func ].dataSize) == 0U )
	return info->ptr = (void *)0;

    info->ptr = new char [ dataSize ];

    RTMemoryUtil::memset( info->ptr, 0, (int)dataSize );

    return info->ptr;
}

static void freeDaemonInfo( TO_daemonInfo * daemon )
{
    freeStr( daemon->invokeID );
}

void TO_freeStructInfo( TO_structInfo * info )
{
    int func;

    if( info == (TO_structInfo *)0 )
	return;

    func = info->func;
    info->func = FUNC_NONE;

    freeDaemonInfo( &info->daemon );

    if( func <= FUNC_NONE || FUNC_LAST <= func )
	return;

    (*theTable[ func ].freePtr)( info );

    if( info->ptr != (void *)0 )
    {
	delete [] (char *)info->ptr;

	info->ptr = (void *)0;
    }
}

int TO_decodeBuffer( TO_bufferInfo * buf, TO_structInfo * info )
{
    if( buf == (TO_bufferInfo *)0 || info == (TO_structInfo *)0 )
	return DEC_BAD_PARM;

    buf->status = DEC_SUCCESS;
    info->func  = FUNC_NONE;
    info->ptr   = (void *)0;

    if( doDecodeFunc( buf, &info->func ) != DEC_SUCCESS )
	return buf->status;

    if( doDecodeDaemonInfo( buf, &info->daemon ) != DEC_SUCCESS )
	return buf->status;

    return (*theTable[ info->func ].decoPtr)( buf, info );
}

/*----------------------------------------------------------------------------
|  Common encoding functions:
+---------------------------------------------------------------------------*/

/*--- Basic encoding - Int, Str, Func: -------------------------------------*/

static int doEncodeFunc( TO_bufferInfo * buf, int func )
{
    const char * funcStr;
    int          funcLen;

    buf->used = 0;

    if( func <= FUNC_NONE || FUNC_LAST <= func )
	return buf->status = ENC_BAD_FUNC_TYPE;

    funcStr = theTable[ func ].funcName;
    funcLen = RTMemoryUtil::strlen( funcStr );

    if( 5 + funcLen > buf->size ) /* 5: "FUNC_" */
	return buf->status = ENC_OVERFLOW;

    RTMemoryUtil::memcpy( buf->data + buf->used, "FUNC_", 5 );
    buf->used = 5;

    RTMemoryUtil::memcpy( buf->data + buf->used, funcStr, funcLen );
    buf->used += funcLen;

    return ENC_SUCCESS;
}

int doEncodeInt( TO_bufferInfo * buf, int intValue )
{
    if( buf->used + 5 > buf->size )
	return buf->status = ENC_OVERFLOW;

    RTMemoryUtil::memcpy( buf->data + buf->used, " INT ", 5 );
    buf->used += 5;

    int intLen = RTFormat::_int( intValue, buf->data + buf->used,
					   buf->size - buf->used );

    if( intLen == 0 )
	return buf->status = ENC_OVERFLOW;

    buf->used += intLen;

    return ENC_SUCCESS;
}

int doEncodeStr( TO_bufferInfo * buf, const char * strValue )
{
    if( buf->used + 5 > buf->size )
	return buf->status = ENC_OVERFLOW;

    RTMemoryUtil::memcpy( buf->data + buf->used, " STR ", 5 );
    buf->used += 5;

    int strLen = strValue != (const char *)0 ? RTMemoryUtil::strlen( strValue )
					     : 0;

    int lenLen = RTFormat::_int( strLen, buf->data + buf->used,
					 buf->size - buf->used );

    if( lenLen == 0 )
	return buf->status = ENC_OVERFLOW;

    buf->used += lenLen;

    if( buf->used + 1 + strLen > buf->size )
	return buf->status = ENC_OVERFLOW;

    buf->data[ buf->used++ ] = ' ';

    if( strLen > 0 )
    {
	//lint -save -e668
	RTMemoryUtil::memcpy( buf->data + buf->used, strValue, strLen );
	//lint -restore
	buf->used += strLen;
    }

    return ENC_SUCCESS;
}

/*--- Semi-special encoding: -----------------------------------------------*/

static int doEncodeDaemonInfo( TO_bufferInfo       * buf,
			       const TO_daemonInfo * daemon )
{
    if( daemon == (const TO_daemonInfo *)0 )
	return buf->status = ENC_BAD_PARM;

    if( doEncodeInt( buf, daemon->number   ) != ENC_SUCCESS ||
	doEncodeStr( buf, daemon->invokeID ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return ENC_SUCCESS;
}

int doEncodeFuncDaemon( TO_bufferInfo       * buf,
			int                   func,
			const TO_daemonInfo * daemon )
{
    return doEncodeFunc      ( buf, func   ) != ENC_SUCCESS ||
	   doEncodeDaemonInfo( buf, daemon ) != ENC_SUCCESS;
}

int doEncodeTimeStamp( TO_bufferInfo * buf, const TO_timeStamp * timestamp )
{
    if( doEncodeInt( buf, (int)timestamp->sec  ) != ENC_SUCCESS ||
	doEncodeInt( buf, (int)timestamp->nsec ) != ENC_SUCCESS )
    {
	return buf->status;
    }

    return ENC_SUCCESS;
}

int doEncodeMessage( TO_bufferInfo * buf, const TO_message * msg )
{
    if( doEncodeStr( buf,                   msg->signal )   != ENC_SUCCESS ||
	doEncodeStr( buf, RTPriority::name( msg->prio   ) ) != ENC_SUCCESS ||
	doEncodeStr( buf,                   msg->data   )   != ENC_SUCCESS )
    {
	return buf->status;
    }

    return ENC_SUCCESS;
}

/*----------------------------------------------------------------------------
|  Common freeing functions:
+---------------------------------------------------------------------------*/

/*--- Generic freeing - Str: -----------------------------------------------*/

void freeStr( char * str )
{
    if( str != (char *)0 )
	delete [] str;
}

} // extern "C"
