/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTAsciiEncoding.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMessage.h>
#include <RTPriority.h>
#include <RTToolsetObserver.h>
#include <RTTruncatingBuffer.h>

int RTToolsetObserver::evtMessageIn( int                  daemonId,
				     const RTDaemonInfo & daemonInfo,
				     const RTEventInfo  & eventInfo )
{
    const RTDaemon * theDaemon = (const RTDaemon *)daemons[ daemonId ];

    if( theDaemon == (const RTDaemon *)0 )
	return 0;

    int shouldHalt = theDaemon->shouldHalt();

    if( theDaemon->shouldReport() == 0 )
	return shouldHalt;

    TO_evtMsgIn   toSendData;
    TO_daemonInfo toDaemonInfo;

    toDaemonInfo.number   = daemonId;
    toDaemonInfo.invokeID = (char *)0;

    toSendData.recTime.sec  = eventInfo.timestamp.tv_sec;
    toSendData.recTime.nsec = eventInfo.timestamp.tv_nsec;

    toSendData.message.signal = eventInfo.signalName;
    toSendData.message.prio   = eventInfo.message->getPriority();

    char                 * msgDataBuf = new char [ (unsigned)msgDataBufSize ];
    const RTObject_class * type;

    if( eventInfo.message->data == (const void *)0 )
	type = (const RTObject_class *)0;
    else
	type = eventInfo.message->getType();

    if( type == (const RTObject_class *)0 )
	msgDataBuf[ 0 ] = '\0';
    else
    {
	RTTruncatingBuffer buffer( msgDataBuf, msgDataBufSize );
	RTAsciiEncoding    coding( &buffer );

	(void)coding.put( eventInfo.message->data, type );
	(void)buffer.write( "", 1 );
    }

    toSendData.message.data = msgDataBuf;

    char * tempActor = buildString( daemonInfo.actorPath,
				    eventInfo.actorIndex );

    toSendData.recActor = tempActor;
    toSendData.recPort  = eventInfo.portName;

    if( TO_encodeEvtMsgIn( &txBuffer, &toDaemonInfo, &toSendData )
			  == ENC_SUCCESS )
    {
	sendTxBuffer();
    }

    delete [] tempActor;
    delete [] msgDataBuf;

    return shouldHalt;
}
