/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTEventInfo_h__
#define __RTEventInfo_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTTimespec_h__
#include <RTTimespec.h>
#endif

class  RTActor;
struct RTActor_class;
class  RTMessage;

class RTEventInfo
{
public:
    enum EventType
    {
	EventPresence,
	EventReference,
	EventChain,
	EventState,
	EventFSM,
	EventLateFSM,
	EventMsgIn,
	EventMsgOut,
	EventMsgOutIn,
	EventTimer,
	EventVariable,
	EventHistory,
	EventControl
    };

    enum
    {
	EventLastEntry = EventControl
    };

    enum SendMethod
    {
	SendNormal,       // normal send
	SendInvoke,       // invoke
	SendReply,        // reply to an invoke

	SendInformIn,     // faked event for informIn
	SendInformAt,     // faked event for informAt
	SendInformEvery   // faked event for informEvery
    };

    // this definition _must_ match the RTDebugger::mode definition
    enum TargetMode
    {
	Halted,
	Stepping,
	Running
    };

			  RTEventInfo( EventType,
				       int        theProbeNr,
				       int        theActorIndex );
			  ~RTEventInfo( void );

    EventType             eventKind;
    int                   probeNr;
    int                   actorIndex;

    int                   observerId;
    int                   daemonId;
    RTTimespec            timestamp;

    // used by chain events:
    char                * chainStrOrigin;
    const char          * chainStrTrans;
    int                   chainOrigin;
    int                   chainPosition;
    int                   chainParent; // only valid and used if origin is a cp

     // used by state and history events:
    char                * stateStrTarget;
    int                   stateTarget;

    char                * portName;
    RTMessage           * message;
    SendMethod            sendMethod;
    const char          * signalName;

    const RTActor       * actor;
    const RTActor_class * info;

    int                   compIndex;
    int                   nrIncarns;

    // used by variable events:
    char                * varName;
    char                * varValue;

    // used by history events:
    char                * histStrHist;
    char                * histStrCurr;
    int                   histHist;
    int                   histCurr;

    // used by control events:
    TargetMode            targetMode;

   // Used for linked event lists
    RTEventInfo         * prev;
    RTEventInfo         * next;

private:
    // unavailable methods
    RTEventInfo( const RTEventInfo & );
    RTEventInfo & operator=( const RTEventInfo & );
}; //lint !e1712

#endif // __RTEventInfo_h__
