/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTTimespec.h>

#include <chorus.h>

static bool use_sysTime;

void RTTimespec_initialize( void )
{
    KnActorPrivilege privilege;

    use_sysTime = true;

    // get the actor's privelege
    if( actorPrivilege( K_MYACTOR, &privilege, 0 ) != K_OK )
	RTDiag::panic( "actorPrivilege" );

    if( privilege == K_SUPACTOR )
    {
	// Supervisor mode actors may use univTimeSet, so we prepare for the
	// possibility that RTTimerSAP::adjustTime(Begin,End) will be used.
	// Make sure that univTime will not fail because univTimeSet has
	// not been called.

	KnTimeVal now;
	KnTimeVal ignored;

	if( univTime( &now ) != K_OK )
	{
	    // Assume this is because the time was never set.
	    // Copy the time from sysTime.

	    if( sysTime( &now ) != K_OK )
		RTDiag::panic( "sysTime" );

	    if( univTimeSet( &ignored, &now ) != K_OK )
		RTDiag::panic( "univTimeSet" );
	}
	use_sysTime = false;
    }
}

void RTTimespec::getclock( RTTimespec & ts )
{
    KnTimeVal now;

    if( use_sysTime )
    {
	if( sysTime( &now ) != K_OK )
	    RTDiag::panic( "sysTime" );
    }
    else
    {
	if( univTime( &now ) != K_OK )
	    RTDiag::panic( "univTime" );
    }

    ts.tv_sec  = now.tmSec;
    ts.tv_nsec = now.tmNSec;
}
