/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTMain.h>

#include <windows.h>
#if OBSERVABLE
#include <ctype.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif

int main( int argc, const char * const * argv )
{
#if OBSERVABLE
    if( argc <= 1 )
    {
	// Attempt to read program arguments from file. This occurs when
	// the toolset is running on NT, with TargetRTS selected, in VC50
	// debugging mode. Note: This is a hack to get around the fact that
	// the VC50 debugger cannot accept program arguments programatically.

	const long     ArgLimit = 32768L;
	const char   * ArgFile  = "TargetRTS.args";
	int            arg_fd   = _open( ArgFile, _O_RDONLY | _O_BINARY );
	struct _stat   info;
	char         * buffer   = (char *)0;
	const char  ** argp     = (const char **)0;

	if( arg_fd == -1 )
	{
	    // can't open file
	}
	else if( _fstat( arg_fd, &info ) == -1 )
	{
	    // can't get info
	}
	else if( info.st_size > ArgLimit )
	{
	    // too much data
	}
	else if( (buffer = new char [ (unsigned)info.st_size + 1U ])
			 == (char *)0 )
	{
	    // can't allocate buffer
	}
	else if( info.st_size != (long)_read( arg_fd,
					      buffer,
					      (unsigned)info.st_size ) )
	{
	    // can't read data
	}
	else
	{
	    buffer[ info.st_size ] = '\0';

	    char * cursor = buffer;

	    argc = 2;
	    for(;;)
	    {
		while( isspace( (int)*cursor ) != 0 )
		    ++cursor;
		if( *cursor == '\0' )
		    break;
		++argc;
		for( bool quoted = false; *cursor != '\0'; ++cursor )
		{
		    if( *cursor == '"' )
			quoted = ! quoted;
		    else if( ! quoted && isspace( (int)*cursor ) != 0 )
			break;
		}
	    }

	    if( (argp = new const char * [ (unsigned)argc ])
		      == (const char **)0 )
	    {
		argc = 1; // not enough memory
		delete [] buffer;
		buffer = (char *)0;
	    }
	    else
	    {
		// we always have 1 arg, the app name
		argp[ 0 ] = argv[ 0 ];

		argc   = 0;
		cursor = buffer;
		for(;;)
		{
		    while( isspace( (int)*cursor ) != 0 )
			++cursor;
		    if( *cursor == '\0' )
			break;
		    argp[ ++argc ] = cursor;

		    char * writec = cursor;
		    for( bool quoted = false; *cursor != '\0'; ++cursor )
		    {
			if( *cursor == '"' )
			    quoted = ! quoted;
			else if( quoted || isspace( (int)*cursor ) == 0 )
			    *writec++ = *cursor;
			else
			    break;
		    }
		    char separator = *cursor++;
		    *writec = '\0';
		    if( separator == '\0' )
			break;
		}
		argp[ ++argc ] = (char *)0;

		argv = argp;
	    }
	}

	if( arg_fd != -1 )
	    (void)_close( arg_fd );

	int rc = RTMain::entryPoint( argc, argv );

	// cleanup

	delete [] argp;
	delete [] buffer;

	return rc;
    }
#endif // OBSERVABLE

    return RTMain::entryPoint( argc, argv );
}
