/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTMain.h"
#pragma implementation "RTMain.inl"
#pragma implementation "RTSystemDescriptor.h"
#endif

#include <RTDiag.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>
#include <RTNucleus.h>
#include <RTSystemDescriptor.h>

struct RTMainThread
{
    int     ( * entry )( void );
    NU_TASK     task;
    char        stack[ NU_MIN_STACK_SIZE ];
};

static void main_thread( UNSIGNED, void * arg )
{
    RTMainThread * info   = (RTMainThread *)arg;
    int            status;

    status = (*info->entry)();

    if( NU_Terminate_Task( NU_Current_Task_Pointer() ) != NU_SUCCESS )
	RTDiag::panic( "terminate(thread/main)" );

    // To release resources associated with this task someone must:
    //
    //  NU_Delete_Task( &info->task );
    //  delete [] (char *)info;
}

int RTMain::execute( void )
{
    int stack_size = _system->mainStackSize;

    if( stack_size < NU_MIN_STACK_SIZE )
	stack_size = NU_MIN_STACK_SIZE;

    int info_size = sizeof( RTMainThread ) - NU_MIN_STACK_SIZE + stack_size;

    RTMainThread * info = (RTMainThread *)new char [ info_size ];

#ifndef NU_NO_ERROR_CHECKING
    RTMemoryUtil::memset( &info->task, 0, (int)sizeof( info->task ) );
#endif

    info->entry = &RTMain::mainLine;

    RTNucleus::Name name;

    RTNucleus::makeName( name, RTNucleus::Thread, (const void *)0 );

    // Create what become the main thread.
    if( NU_Create_Task( &info->task,
			name,
			main_thread,
			1U, info,
			&info->stack,
			stack_size,
			(OPTION)_system->mainPriority,
			0U, // no time slice
			NU_PREEMPT,
			NU_START ) != NU_SUCCESS )
	RTDiag::panic( "create(thread/main)" );

    return 0;
}
