/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTOse_h__
#define __RTOse_h__ included

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

#include <ose.h>
#include <ose_err.h>
#include <inet.h>
#include <ineterr.h>

#if defined( softOSE ) && defined( sun )
    // We need netdb.h for the definition of hostent, but on Solaris
    // that file includes sys/types.h and netinet/in.h which clash
    // with header files provided by OSE. These macros prevent the
    // inclusion of those header files.
#   define _SYS_TYPES_H
#   define _NETINET_IN_H

    // netdb.h needs in_addr_t
    typedef struct in_addr in_addr_t;
#endif

#include <netdb.h>

extern "C" hostent * gethostbyname( const char * );

// Undefine macros that OSE defines which conflict with our function names.

#undef accept
#undef bind
#undef close
#undef connect
#undef listen
#undef read
#undef select
#undef send
#undef shutdown
#undef start
#undef stop
#undef write

// Replace OSE macros with names that do not conflict with common signals.

#if ! defined( OS_DEBUG )
# if defined( softOSE )
#   define ose_send( sig, to ) zzzsend( (sig), (to) )
# else
#   define ose_send( sig, to ) send( (sig), (to) )
# endif
#   define ose_start( id )     start( (id) )
#   define ose_stop( id )      stop( (id) )
#elif defined( ZZFILE_AT_HEAD )
#   define ose_send( sig, to ) zzsend( __FILE__, __LINE__, (sig), (to) )
#   define ose_start( id )     zzstart( __FILE__, __LINE__, (id) )
#   define ose_stop( id )      zzstop( __FILE__, __LINE__, (id) )
#else
#   define ose_send( sig, to ) zzsend( (sig), (to), __FILE__, __LINE__ )
#   define ose_start( id )     zzstart( (id), __FILE__, __LINE__ )
#   define ose_stop( id )      zzstop( (id), __FILE__, __LINE__ )
#endif

// Signal definitions for TargetRTS.

enum
{
    RTOseSyncSignal  = 1
  , RTOseStartSignal
};

struct RTSyncData
{
    SIGSELECT _number;
};

class  RTJob;
struct RTThreadInfo;

struct RTStartData
{
    SIGSELECT      _number;
    RTJob        * _job;
    RTThreadInfo * _info;
};

union SIGNAL
{
    SIGSELECT   _number;
    RTSyncData  _sync;
    RTStartData _start;
};

#endif // __RTOse_h__
