/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTObsCtrlTask.h"
#endif

#include <RTDiag.h>
#include <RTDebugger.h>
#include <RTIOMonitor.h>
#include <RTMain.h>
#include <RTObsCtrlTask.h>
#include <RTinet.h>
#include <RTpSOS.h>
#include <RTtcp.h>

#include <psos.h>

void RTObsCtrlTask::sendEvent( void )
{
    ev_send( tid, WakeupEvent );
}

void RTObsCtrlTask::recvEvents( RTObsCtrlTask * task )
{
    for(;;)
    {
	unsigned long events;
	static char   anything = 'C';

	if( ev_receive( WakeupEvent | ShutdownEvent,
			EV_WAIT | EV_ANY, 0, &events ) )
	    RTDiag::panic( "ev_receive(obs)" );

	if( ( events & ShutdownEvent ) != 0 )
	{
	    RTinet_close( task->send_fd );
	    task->send_fd = INET_NO_SOCKET;
	    sm_v( task->sem_id );
	    break;
	}

	if( RTinet_write( task->send_fd, &anything, sizeof( anything ) ) < 0 )
	    RTDiag::panic( "write(obs)" );
    }

    rt_thread_exit();
}

RTObsCtrlTask::RTObsCtrlTask( int _socket )
    : tid    (              0 )
    , send_fd( INET_NO_SOCKET )
    , sem_id (              0 )
{
    unsigned long arg  = (unsigned long)this;
    unsigned long prio;

    if( t_setpri( 0, RTMaxPriority, &prio ) )
	RTDiag::panic( "t_setpri(obs,1)" );

    else if( t_setpri( 0, prio, &prio ) )
	RTDiag::panic( "t_setpri(obs,2)" );

    if( t_create( "OBSt", prio, 4096, 0, T_LOCAL | T_FPU, &tid ) )
	RTDiag::panic( "t_create(obs)" );

    else if( (send_fd = shr_socket( _socket, (int)tid )) == INET_NO_SOCKET )
	RTDiag::panic( "shr_socket(obs)" );

    else if( t_start( tid, T_PREEMPT | T_TSLICE | T_NOASR | T_SUPV,
		      (void (*)( void ))recvEvents, &arg ) )
	RTDiag::panic( "t_start(obs)" );
}

RTObsCtrlTask::~RTObsCtrlTask( void )
{
    // signal our helper task to shutdown
    ev_send( tid, ShutdownEvent );

    // wait for it to finish
    sm_p( sem_id, SM_WAIT, 0 );

    if( sm_delete( sem_id ) != 0 )
	RTDiag::panic( "sm_delete(obs)" );
}

void RTDebugger::startupUdp( void )
{
    RTinet_address address;
    RTinet_port    port;

    if( (_udp = RTinet_UDP_create()) == INET_NO_SOCKET )
	RTDiag::panic( "UDP_create(obs)" );

    else
    {
	if( ! RTinet_bind( _udp, (const RTinet_address *)0, 0 ) )
	    RTDiag::panic( "bind(obs)" );

	else if( ! RTinet_getname( _udp, &address, &port ) )
	    RTDiag::panic( "getsockname(obs)" );

	else if( RTinet_connect( _udp, &RTinet_loopback, port ) < 0 )
	    RTDiag::panic( "connect(obs)" );

	else
	{
	    ioMonitor->add( _udp, RTIOMonitor::CanRead );
	    _trigger = new RTObsCtrlTask( _udp );
	    return;
	}

	//lint -unreachable
	RTinet_close( _udp );
	_udp = INET_NO_SOCKET;
    }
}
