/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTTimespec.h>

#include <psos.h>
#include <configs.h>

extern "C" NodeConfigTable * anchor;

// Number of days in a non-leap year preceding the first of the month.

static const unsigned short startOfMonth[] =
{
    /* Jan */   0,
    /* Feb */   31,
    /* Mar */   31 + 28,
    /* Apr */   31 + 28 + 31,
    /* May */   31 + 28 + 31 + 30,
    /* Jun */   31 + 28 + 31 + 30 + 31,
    /* Jul */   31 + 28 + 31 + 30 + 31 + 30,
    /* Aug */   31 + 28 + 31 + 30 + 31 + 30 + 31,
    /* Sep */   31 + 28 + 31 + 30 + 31 + 30 + 31 + 31,
    /* Oct */   31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30,
    /* Nov */   31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31,
    /* Dec */   31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30
};

static bool initialized   = false;
static long nsec_per_tick;

void RTTimespec::getclock( RTTimespec & ts )
{
    if( ! initialized )
    {
	nsec_per_tick = 1000000000L / anchor->psosct->kc_ticks2sec;
	initialized   = true;
    }

    unsigned long date;
    unsigned long time;
    unsigned long ticks;

    if( tm_get( &date, &time, &ticks ) == ERR_NOTIME )
    {
	tm_set( ( 1970 << 16 ) + ( 1 << 8 ) + 1, 0, 0 );
	tm_get( &date, &time, &ticks );
    }

    unsigned long year   = ( ( date >> 16 ) & 0xFFFF );
    unsigned long month  = ( ( date >>  8 ) & 0x00FF ) - 1;
    unsigned long day    = ( ( date       ) & 0x00FF ) - 1;
    unsigned long hour   = ( ( time >> 16 ) & 0xFFFF );
    unsigned long minute = ( ( time >>  8 ) & 0x00FF );
    unsigned long second = ( ( time       ) & 0x00FF );
    int           leap;

    if     ( ( year %   4 ) != 0 ) leap = 0;
    else if( ( year % 100 ) != 0 ) leap = 1;
    else if( ( year % 400 ) != 0 ) leap = 0;
    else                           leap = 1;

    // The choice of January 1, 1970 as the time base is arbitrary
    // but will put pSOS on an equal footing with other operating
    // systems with respect to range of time values: the seconds
    // component turns negative on January 19, 2038 at 03:14:08.
    // and wraps to zero on February 7, 2106 at 06:28:16.

    // units in tv_sec are days
    ts.tv_sec  = ( year - 1970 ) * 365;
    ts.tv_sec += ( year - 1969 ) /   4; // primary   leap-year adjustment
    ts.tv_sec -= ( year - 1901 ) / 100; // secondary leap-year adjustment
    ts.tv_sec += ( year - 1601 ) / 400; // tertiary  leap-year adjustment

    ts.tv_sec += startOfMonth[ month ] + ( leap && month > 1 );
    ts.tv_sec += day;

    ts.tv_sec *= 24; // change units to hours
    ts.tv_sec += hour;

    ts.tv_sec *= 60; // change units to minutes
    ts.tv_sec += minute;

    ts.tv_sec *= 60; // change units to seconds
    ts.tv_sec += second;

    ts.tv_nsec = ticks * nsec_per_tick;
}
