/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTAbortController.h>
#include <RTDiag.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>
#include <RTWinCE.h>

static void abortApplication( RTWinCE * ce )
{
    if( ce->mainThread == (HANDLE)0 )
	return; // there is no main thread

    // If the main thread still exists, suspend it so it
    // doesn't change the state of the main controller.

    if( SuspendThread( ce->mainThread ) != (DWORD)-1 )
    {
	RTController * mainController = RTMain::controller();

	if( mainController != (RTController *)0 )
	{
	    RTAbortController abort( mainController->debugger() );

	    abort.mainLoop();
	}

	ResumeThread( ce->mainThread );
    }

    for( MSG msg; ; )
    {
	// Check if main thread had terminated.
	DWORD waitResult = WaitForSingleObject( ce->mainThread, (DWORD)10 );

	if( waitResult == WAIT_OBJECT_0 )
	    break;
	else if( waitResult == WAIT_FAILED )
	    RTDiag::panic( "mainThread(wait)" );

	// Handle any messages from the application as it shuts down.
	if( PeekMessage( &msg, (HWND)0, 0, 0, PM_REMOVE ) )
	{
	    TranslateMessage( &msg );
	    DispatchMessage( &msg );
	}
    }

    if( CloseHandle( ce->mainThread ) == 0 )
	RTDiag::panic( "mainThread(close)" );

    ce->mainThread = (HANDLE)0;
}

LRESULT RTConsole::handleMessage( UINT message, WPARAM wParam, LPARAM lParam )
{
    switch( message )
    {
    case WM_SIZE:
	resize();
	break;

    case WM_PAINT:
	paint();
	break;

    case WM_SETFOCUS:
	haveFocus = 1;
	makeCaret( CaretVisible );
	break;

    case WM_KILLFOCUS:
	haveFocus = 0;
	makeCaret( CaretDestroyed );
	break;

    case WM_CHAR:
	enqueueChar( (TCHAR)( wParam & 0xFF ) );
	break;

    case WM_VSCROLL:
	switch( LOWORD( wParam ) )
	{
	case SB_LINEUP:     scroll(              -1, 0 ); break;
	case SB_LINEDOWN:   scroll(               1, 0 ); break;
	case SB_PAGEUP:     scroll( 1 - screen.rows, 0 ); break;
	case SB_PAGEDOWN:   scroll( screen.rows - 1, 0 ); break;
	case SB_TOP:        scroll(   - buffer.rows, 0 ); break;
	case SB_BOTTOM:     scroll(     buffer.rows, 0 ); break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
	    scroll( HIWORD( wParam ) - screen.top, 0 );
	    break;
	case SB_ENDSCROLL:
	    break;
	}
	break;

    case WM_HSCROLL:
	switch( LOWORD( wParam ) )
	{
	case SB_LINELEFT:   scroll( 0,              -1 ); break;
	case SB_LINERIGHT:  scroll( 0,               1 ); break;
	case SB_PAGELEFT:   scroll( 0, 1 - screen.cols ); break;
	case SB_PAGERIGHT:  scroll( 0, screen.cols - 1 ); break;
	case SB_LEFT:       scroll( 0,   - buffer.cols ); break;
	case SB_RIGHT:      scroll( 0,     buffer.cols ); break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
	    scroll( 0, HIWORD( wParam ) - screen.left );
	    break;
	case SB_ENDSCROLL:
	    break;
	}
	break;

    case WM_CLOSE:
    {
	static const char quit[] = "\nquit\n";

	input_mutex.enter();

	input.lines = 2;
	input.head  = 0;
	input.tail  = (int)sizeof( quit ) - 1;
	RTMemoryUtil::memcpy( input.data, quit, input.tail );

	input_mutex.leave();

	input_available.signal();

	abortApplication( RTWinCE::soleInstance );
	SetWindowLong( window, 0, 0L );
	DestroyWindow( window );
	delete this;
	break;
    }

    case WM_OutputString:
	outputString( (const char *)lParam );
	break;

    default:
	return DefWindowProc( window, message, wParam, lParam );
    }

    return 0;
}
