/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTWinCE_h__
#define __RTWinCE_h__ included

#ifndef __RTMutex_h__
#include <RTMutex.h>
#endif

#ifndef __RTSyncObject_h__
#include <RTSyncObject.h>
#endif

#include <windows.h>
#include <winuser.h>

// When running in the emulation environment,
// we must use thread priorities as defined by NT.
#if defined( _WIN32_WCE_EMULATION )
#undef  THREAD_PRIORITY_NORMAL
#define THREAD_PRIORITY_NORMAL 0
#endif

#if USE_WINCE_RTCONSOLE

class RTConsole
{
public:
    explicit    RTConsole( HWND, int rows = 24, int cols = 80 );
		~RTConsole( void );

    static int  readChar( void );
    static void writeString( const char * );

    static LRESULT CALLBACK WndProc( HWND, UINT message, WPARAM, LPARAM );

private:
    static RTConsole * soleInstance;

    enum CaretState
    {
	CaretDestroyed
      , CaretHidden
      , CaretVisible
    };

    enum
    {
	WM_OutputString = WM_USER + 0
    };

    LRESULT         handleMessage( UINT message, WPARAM, LPARAM );

    void            enqueueChar( TCHAR );
    void            makeCaret( CaretState );
    void            outputString( const char * );
    void            paint( void );
    void            resize( void );
    void            scroll( int rows, int cols );

    HWND            window;
    HFONT           font;
    int             haveFocus;
    CaretState      caretState;

    RTMutex         input_mutex;
    RTSyncObject    input_available;

    struct
    {
	enum { Size = 256 };

	int         lines;
	int         head;
	int         tail;
	char        data[ Size ];
    } input;

    struct
    {
	TCHAR     * data;
	int         rows; // number of rows in the buffer
	int         cols; // number of columns in the buffer
    } buffer;

    struct
    {
	int         top;  // Top-most row visible on the screen
	int         left; // Left-most column visible on the screen
	int         rows; // Number of rows visible on the screen
	int         cols; // Number of columns visible on the screen
    } screen;

    struct
    {
	int         row;
	int         col;
    } cursor;

    struct
    {
	int         height;
	int         width;
    } character;

    // unavailable methods
    RTConsole( const RTConsole & );
    RTConsole & operator=( const RTConsole & );
};

#define IDR_MAIN_MENU 101

struct RTWinCE
{
//  HWND                hMainWnd;
    HWND                hCmdBar;
    HWND                hTermWnd;

    HANDLE              mainThread;

    static RTWinCE    * soleInstance;
};

#endif // USE_WINCE_RTCONSOLE

#endif // __RTWinCE_h__
